=== modified file 'bzrlib/transport/chroot.py'
--- bzrlib/transport/chroot.py	2009-03-23 14:59:43 +0000
+++ bzrlib/transport/chroot.py	2009-09-16 04:37:33 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,21 +17,18 @@
 """Implementation of Transport that prevents access to locations above a set
 root.
 """
-from urlparse import urlparse
 
-from bzrlib import errors, urlutils
 from bzrlib.transport import (
     get_transport,
+    pathfilter,
     register_transport,
     Server,
     Transport,
     unregister_transport,
     )
-from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
-from bzrlib.transport.memory import MemoryTransport
-
-
-class ChrootServer(Server):
+
+
+class ChrootServer(pathfilter.PathFilteringServer):
     """User space 'chroot' facility.
 
     The server's get_url returns the url for a chroot transport mapped to the
@@ -39,126 +36,40 @@
     directories of the backing transport are not visible. The chroot url will
     not allow '..' sequences to result in requests to the chroot affecting
     directories outside the backing transport.
+
+    PathFilteringServer does all the path sanitation needed to enforce a
+    chroot, so this is a simple subclass of PathFilteringServer that ignores
+    filter_func.
     """
 
     def __init__(self, backing_transport):
-        self.backing_transport = backing_transport
+        pathfilter.PathFilteringServer.__init__(self, backing_transport, None)
 
     def _factory(self, url):
         return ChrootTransport(self, url)
 
-    def get_url(self):
-        return self.scheme
-
     def setUp(self):
         self.scheme = 'chroot-%d:///' % id(self)
         register_transport(self.scheme, self._factory)
 
-    def tearDown(self):
-        unregister_transport(self.scheme, self._factory)
-
-
-class ChrootTransport(Transport):
+
+class ChrootTransport(pathfilter.PathFilteringTransport):
     """A ChrootTransport.
 
     Please see ChrootServer for details.
     """
 
-    def __init__(self, server, base):
-        self.server = server
-        if not base.endswith('/'):
-            base += '/'
-        Transport.__init__(self, base)
-        self.base_path = self.base[len(self.server.scheme)-1:]
-        self.scheme = self.server.scheme
-
-    def _call(self, methodname, relpath, *args):
-        method = getattr(self.server.backing_transport, methodname)
-        return method(self._safe_relpath(relpath), *args)
-
-    def _safe_relpath(self, relpath):
-        safe_relpath = self._combine_paths(self.base_path, relpath)
-        if not safe_relpath.startswith('/'):
-            raise ValueError(safe_relpath)
-        return safe_relpath[1:]
-
-    # Transport methods
-    def abspath(self, relpath):
-        return self.scheme + self._safe_relpath(relpath)
-
-    def append_file(self, relpath, f, mode=None):
-        return self._call('append_file', relpath, f, mode)
-
-    def _can_roundtrip_unix_modebits(self):
-        return self.server.backing_transport._can_roundtrip_unix_modebits()
-
-    def clone(self, relpath):
-        return ChrootTransport(self.server, self.abspath(relpath))
-
-    def delete(self, relpath):
-        return self._call('delete', relpath)
-
-    def delete_tree(self, relpath):
-        return self._call('delete_tree', relpath)
-
-    def external_url(self):
-        """See bzrlib.transport.Transport.external_url."""
-        # Chroots, like MemoryTransport depend on in-process
-        # state and thus the base cannot simply be handed out.
-        # See the base class docstring for more details and
-        # possible directions. For now we return the chrooted
-        # url.
-        return self.server.backing_transport.external_url()
-
-    def get(self, relpath):
-        return self._call('get', relpath)
-
-    def has(self, relpath):
-        return self._call('has', relpath)
-
-    def is_readonly(self):
-        return self.server.backing_transport.is_readonly()
-
-    def iter_files_recursive(self):
-        backing_transport = self.server.backing_transport.clone(
-            self._safe_relpath('.'))
-        return backing_transport.iter_files_recursive()
-
-    def listable(self):
-        return self.server.backing_transport.listable()
-
-    def list_dir(self, relpath):
-        return self._call('list_dir', relpath)
-
-    def lock_read(self, relpath):
-        return self._call('lock_read', relpath)
-
-    def lock_write(self, relpath):
-        return self._call('lock_write', relpath)
-
-    def mkdir(self, relpath, mode=None):
-        return self._call('mkdir', relpath, mode)
-
-    def open_write_stream(self, relpath, mode=None):
-        return self._call('open_write_stream', relpath, mode)
-
-    def put_file(self, relpath, f, mode=None):
-        return self._call('put_file', relpath, f, mode)
-
-    def rename(self, rel_from, rel_to):
-        return self._call('rename', rel_from, self._safe_relpath(rel_to))
-
-    def rmdir(self, relpath):
-        return self._call('rmdir', relpath)
-
-    def stat(self, relpath):
-        return self._call('stat', relpath)
+    def _filter(self, relpath):
+        # A simplified version of PathFilteringTransport's _filter that omits
+        # the call to self.server.filter_func.
+        return self._relpath_from_server_root(relpath)
 
 
 class TestingChrootServer(ChrootServer):
 
     def __init__(self):
         """TestingChrootServer is not usable until setUp is called."""
+        ChrootServer.__init__(self, None)
 
     def setUp(self, backing_server=None):
         """Setup the Chroot on backing_server."""
@@ -171,5 +82,4 @@
 
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    return [(ChrootTransport, TestingChrootServer),
-            ]
+    return [(ChrootTransport, TestingChrootServer)]

=== modified file 'bzrlib/transport/pathfilter.py'
--- bzrlib/transport/pathfilter.py	2009-09-14 05:39:09 +0000
+++ bzrlib/transport/pathfilter.py	2009-09-16 04:37:33 +0000
@@ -81,15 +81,14 @@
         self.base_path = self.base[len(self.server.scheme)-1:]
         self.scheme = self.server.scheme
 
-    def _filtered(self):
-        return self.server.backing_transport.clone(
-            self._filter(self.base_path))
-        
-    def _filter(self, relpath):
+    def _relpath_from_server_root(self, relpath):
         unfiltered_path = self._combine_paths(self.base_path, relpath)
         if not unfiltered_path.startswith('/'):
             raise ValueError(unfiltered_path)
-        return self.server.filter_func(unfiltered_path[1:])
+        return unfiltered_path[1:]
+
+    def _filter(self, relpath):
+        return self.server.filter_func(self._relpath_from_server_root(relpath))
 
     def _call(self, methodname, relpath, *args):
         """Helper for Transport methods of the form:
@@ -100,19 +99,12 @@
 
     # Transport methods
     def abspath(self, relpath):
-        # I think we do *not* want to filter at this point; e.g if the filter
-        # is homedir expansion, self.base == 'this:///' and relpath == '~/',
-        # then the abspath should be this:///~/foo.  Instead filtering should
-        # happen when self's base is passed to the backing.
-        # On the other hand, this means that a relpath of '../../..' will not
-        # be an error here, but I think that's ok (or even good?)
-        full_path = self._combine_paths(self.base_path, relpath)
-        if not full_path.startswith('/'):
-            raise AssertionError(
-                '_combine_paths did not return a path starting with / '
-                '(got %r)' % (full_path,))
-        path = full_path[1:]
-        return self.scheme + path
+        # We do *not* want to filter at this point; e.g if the filter is
+        # homedir expansion, self.base == 'this:///' and relpath == '~/foo',
+        # then the abspath should be this:///~/foo (not this:///home/user/foo).
+        # Instead filtering should happen when self's base is passed to the
+        # backing.
+        return self.scheme + self._relpath_from_server_root(relpath)
 
     def append_file(self, relpath, f, mode=None):
         return self._call('append_file', relpath, f, mode)
@@ -121,7 +113,7 @@
         return self.server.backing_transport._can_roundtrip_unix_modebits()
 
     def clone(self, relpath):
-        return PathFilteringTransport(self.server, self.abspath(relpath))
+        return self.__class__(self.server, self.abspath(relpath))
 
     def delete(self, relpath):
         return self._call('delete', relpath)
@@ -147,7 +139,9 @@
         return self.server.backing_transport.is_readonly()
 
     def iter_files_recursive(self):
-        return self._filtered().iter_files_recursive()
+        backing_transport = self.server.backing_transport.clone(
+            self._filter('.'))
+        return backing_transport.iter_files_recursive()
 
     def listable(self):
         return self.server.backing_transport.listable()

